-- rotating_lena.love

controls  = require 'mouse_control'
scrollbar = require 'scrollbar'

local gr, mg, ky		= love.graphics, love.image, love.keyboard
local tm, ev, lm 		= love.timer, love.event, love.mouse
local pi, sin, cos		= math.pi, math.sin, math.cos

function love.load()
	load_screen_settings()
	create_quads()
	create_spritebatch()
	load_effect()
	load_options()
	end

function love.draw()
	-- center spritebatch (calculate offsets)
	sbw, sbh = quad.steps.x * ( quad.size.x + sp ) / 2, quad.steps.y * ( quad.size.y + sp ) / 2
	gr.setPixelEffect( effect )
		gr.draw( spritebatch, image.x, image.y, image.angle, image.zoom, image.zoom, sbw, sbh )
		gr.setPixelEffect( )
	gr.setColor( 185, 185,   0, 255 )
	gr.print( 'fps: '..tm.getFPS( ), 10,  535 )
	gr.print( 'number of quads: '..quad.steps.x * quad.steps.y, 10,  555 )
	gr.print( '<c> for mouse controls', 10, 575 )
	gr.draw( ball, width - ball_size, height - ball_size, 0, 1, 1, ball_size, ball_size )
	gr.setColor(   0,   0, 180, 255 )
	gr.print('R',width-ball_size-controls.fontsize/2+2,height-ball_size-controls.fontsize/2)
	if controls.display  then controls:show()  end
	if scrollbar.display then scrollbar:show() end
	end

function love.update( dt )
	image.step = dt
	if image.rotate then
		image.angle = image.angle > 2 * pi and 0 or image.angle + image.rate * pi / 360		-- image rotation
		else
		image.angle = 0
		end
	if light.rotate then
		light.angle	= light.angle > 0  and light.angle - 2 * dt * light.rate or 2 * pi		-- light rotation
		loc = {
			light.center.x + light.radius * sin( -light.angle ),
			light.center.y + light.radius * cos( -light.angle ) }
		else
		loc = { light.x, light.y }
		end
	set_effect( loc, light.d )
	end

function love.keypressed( key )
	if key == 'escape'		then ev.push( 'quit' )								end
	if key == 'c'			then	controls.display  = not controls.display
									scrollbar.display = not scrollbar.display	end
	end

function load_screen_settings()
	gr.setCaption( "A test of SpriteBatch" )
	width, height	= 800, 600
	gr.setMode( width, height )
	gr.setBackgroundColor( 10,10,10,255)
	image				= {
		image			= gr.newImage( 'lenaC.jpg' ),
		angle			= 0,
		step			= 0,
		rotate			= false,
		x				= width,
		y				= height
						}
	quad				= {
		width			= image.image:getWidth(),
		height			= image.image:getHeight(),
		images			= {},
		size			= { x = 4, y = 4 }}
	quad.steps			= { x = quad.width / quad.size.x,
							y = quad.height / quad.size.y }
	set_defaults()
	create_ball()
	end

function load_options()
	controls.table = {
		{'Image',			function() 										end },
		{' - zoom',			function() sel = 1								end },
		{' - rotate',		function() image.rotate	= true 	sel = 2			end },
		{' - left/right',	function() sel = 3								end },
		{' - up/down',		function() sel = 4								end },
		{' - explode',		function() sel = 5								end },
		{'Light',			function()										end },
		{' - diameter',		function() sel = 6								end },
		{' - rotate',		function() light.rotate	= true	sel = 7			end },
		{'    - radius',	function() light.rotate = true	sel = 8			end },
		{' - left/right',	function() sel = 9								end },
		{' - up/down',		function() sel = 10								end },
		{'Reset',			function() set_defaults() create_spritebatch()	end },
		{'Exit',			function() ev.push( 'quit' )					end },
		}
	scrollbar.table = {
		{'zoom',				function(val) image.zoom = val					 end, min =   -5, max =    5, cur = 0.9 },
		{'image rotation', 		function(val) image.rate = val image.rotate=true end, min =  -20, max =   20, cur = 0   },
		{'image left & right',	function(val) image.x	 = val					 end, min =    0, max =  800, cur = 400 },
		{'image up & down',		function(val) image.y	 = val					 end, min =  600, max =    0, cur = 300 },
		{'explode',				function(val) sp = val create_spritebatch()		 end, min = -150, max =  150, cur =   0 },
		{'light diameter',		function(val) light.d	 = val					 end, min =    0, max =    2, cur = 0.25},
		{'light rotation',		function(val) light.rate = val light.rotate=true end, min =  -20, max =   20, cur = 0   },
		{'light orbit radius',	function(val) light.radius=val light.rotate=true end, min =    0, max =  0.5, cur = 0.25},
		{'light left & right',	function(val) light.x = val	light.rotate=false	 end, min = 0.25, max = 0.75, cur = 0.5 },
		{'light up & down',		function(val) light.y = val	light.rotate=false	 end, min = 0.75, max = 0.25, cur = 0.5 },
		}
	--love._openConsole()
	sel = 5										-- default scrollbar display (explode)
	end

function set_defaults()							-- used by load & reset
	image.rotate 		= true
	image.rate			= 0
	image.x, image.y	= width / 2, height / 2
	image.angle			= 0
	image.zoom			= 0.9
	sp					= 0						-- space between quads on screen
	light 				= {
		rotate			= false,
		rate			= 0,
		x				= 0.5,
		y				= 0.5,
		d				= 0.25,
		radius			= 0.25,
		angle			= 0,
		center			= { x = 0.5, y = 0.5 }
						}
	end

function create_quads()				-- table of quads
	for j = 1, quad.steps.y do
		quad.images[j] = {}
		for i = 1, quad.steps.x do
			quad.images[j][i] = gr.newQuad(	quad.size.x * (i-1),	quad.size.y * (j-1),
									quad.size.x,			quad.size.y,
									quad.width,				quad.height )
			end
		end
	end

function create_spritebatch()											-- create sprite batch
	spritebatch = gr.newSpriteBatch( image.image, quad.steps.x * quad.steps.y )
	spritebatch:clear()
	for j = 1, quad.steps.y do
		for i = 1, quad.steps.x do
			spritebatch:addq(
					quad.images[j][i],									-- quad to add
					(quad.size.x+sp) * (i-1), (quad.size.y+sp) * (j-1),	-- screen position
					0,													-- quad rotation
					1,1,												-- quad size
					quad.size.x,quad.size.y 							-- quad offsets
					)
			end
		end
	end

function set_effect( xy, d )									-- send settings to shader
	effect:send( 'pos', xy )
	effect:send( 'd',d )
	end

function load_effect()
	effect = love.graphics.newPixelEffect
		[[
		extern float d = 1.0;						// diameter of spot light
		extern vec2 pos = vec2(0.5,0.5);			// location of spot light
		vec4 effect(vec4 global_color, Image texture, vec2 tc, vec2 pixel_coords)
		{
			vec4 pixel	= Texel( texture, tc );
			float d2 	= clamp(d,0,10);
			float root	= sqrt((pos[0]-tc.x)*(pos[0]-tc.x)+(pos[1]-tc.y)*(pos[1]-tc.y))/d2;
			pixel.r		= pixel.r - root;
			pixel.g		= pixel.g - root;
			pixel.b		= pixel.b - root;
			if (pixel.a > 0.0 ) { pixel.a = 1 - root; }
			return  pixel;
			}
		]]
	end

function create_ball()
	ball_size	= 2 * controls.fontsize
	ball		= love.image.newImageData( 2 * ball_size, 2 * ball_size )
	ball:mapPixel( function( x, y )
		x		= ( x - ball_size ) / ball_size
		y		= ( y - ball_size ) / ball_size
		local d = math.sqrt( x * x + y * y )
		return 255, 255, 255, 255 - math.min( 1, math.pow( d, 0.7 ) ) * 255
		end )
	ball		= love.graphics.newImage( ball )
	end



