---------------------------------------------------------------
-- ** functions for mouse selection of control parameters ** --
---------------------------------------------------------------
-- calling script must contain:
	-- controls = require 'mouse_control'
	-- controls:table
	-- controls:show()
-- syntax for controls.table: {'control name', function() control_activity end }

local controls = {			-- default settings (can be overwritten in 'main')
	x 			=   5,		-- default location - along left side of screen
	y			=   2,
	width		= 100,
	height		=  21,
	spacing		=   0,
	fontsize	=  17,
	font 		= love.graphics.newFont( 17 ),
	table		= {},
	color		= {
		text	= {  10, 180,   0, 255 },
		over	= { 180, 180,   0, 255 },
		sel		= { 250,   0,   0, 255 },
		shadow	= { 100,  70, 110, 255 },
				},
	shadow		= false,
	title		= 'Controls',
	display		= true
	}

function controls:show()
	love.graphics.setFont( self.font )
	if self.shadow then					-- provide drop shadow (if set)
		love.graphics.setColor( self.color.shadow )
		love.graphics.print( self.title, self.x + 6, self.y + 6 )
		end
	love.graphics.setColor( self.color.text )
	love.graphics.print( self.title, self.x + 5, self.y + 5 )
	for box = 1, #self.table do
		--love.graphics.rectangle( 'line', self.x, self.y + box * ( self.spacing + self.height ),
			--	self.width, self.height )
		if self.shadow then
			love.graphics.setColor( self.color.shadow )
			love.graphics.print( self.table[box][1],
			self.x + 6,
			self.y + box * (self.spacing+self.height) + (self.height-self.fontsize)/2+1 )
			end
		love.graphics.setColor( controls.color.text )
		love.graphics.print( self.table[box][1],
			self.x + 5,
			self.y + box * (self.spacing+self.height) + (self.height-self.fontsize)/2 )
		end
	controls:mouse()	-- see if mouse active
	end

function controls:mouse_over_control( x, y, i )	-- check for box contact
	local xmin	= self.x
	local xmax	= self.x + self.width
	local ymin	= self.y + i * ( self.spacing + self.height )
	local ymax	= self.y + i * ( self.spacing + self.height ) + self.height
	return x > xmin and x < xmax and y > ymin and y < ymax
	end

function controls:mouse()
	local x, y = love.mouse.getPosition( )
	if x >= controls.x and x<=controls.x + controls.width then	-- only scan if possibly over boxes
		for box = 1, #self.table do					-- change color, add box, take action
			if self:mouse_over_control( x, y, box ) then
				if love.mouse.isDown( 'l' ) then	-- mouse over box & left clicked
					love.graphics.setColor( self.color.sel )
					self.table[box][2]()			-- perform action
					else love.graphics.setColor( self.color.over )
					end
				love.graphics.rectangle( 'line', self.x, self.y + box * ( self.spacing + self.height ),
					self.width, self.height )
				love.graphics.print( self.table[box][1],
					self.x + 5,
					self.y + box*(self.spacing+self.height) + (self.height-self.fontsize)/2)
				end
			end
		end
	end

return controls
