-----------------------------------------
-- ** strollbar controlled by mouse ** --
-----------------------------------------
-- calling script must contain:
	-- scrollbar = require 'scrollbar'
	-- scrollbar:table
	-- scrollbar:show()
-- scrollbar.table syntax: {'text', function() control_activity end, min, max, cur }
-- Note: min & max are actually the values for the left & right ends of the scrollbar
local gr			= love.graphics
local heigth		= gr.getHeight()	-- screen height
local scrollbar		= {					-- default settings (can be overwritten in 'main')
		x			= 300,				-- bar location
		y			= heigth - 30,
		thickness	= 4,				-- thickness of scrollbar
		length		= 300,				-- length of scrollbar
		table		= {},				-- table containing parameter name, limits & value
		display		= true,
		fontsize	= 17,
		font 		= love.graphics.newFont( 17 ),
		color		= {
			over	= {   0, 185,   0, 255 },	-- mouse over
			sel		= { 155,   0,   0, 255 },	-- slider left-clicked
			slide	= { 185, 185,   0, 255 },	-- inactive slider
			base	= {  75,  75, 150, 255 },	-- scrollbar color
						}
					}
function scrollbar:mouse_over_slider( x, y )	-- check for slider contact
		local sx = self.x + self.length * (self.table[sel].cur-self.table[sel].min) / ( self.table[sel].max-self.table[sel].min)
		local xmin		= sx 		- self.thickness
		local xmax		= sx		+ self.thickness
		local ymin		= self.y	- 2 * self.thickness
		local ymax		= self.y	+ self.thickness
		return x > xmin and x < xmax and y > ymin and y < ymax
		end

function scrollbar:show()
	love.graphics.setFont( self.font )
	-- draw scroll bar (without slider)
	gr.setColor( self.color.base )
	gr.rectangle( 'fill', self.x, self.y - self.thickness, self.length, self.thickness )
	gr.rectangle( 'fill', self.x, self.y - 2*self.thickness, self.thickness, 3*self.thickness )
	gr.rectangle( 'fill', self.x + self.length,self.y-2*self.thickness,-self.thickness,3*self.thickness)
	-- draw slider (color = default, over or selected)
	gr.setColor( self:mouse() )
	local sx = self.x + self.length * (self.table[sel].cur-self.table[sel].min) / ( self.table[sel].max-self.table[sel].min)
	gr.rectangle(
		'fill', sx - self.thickness/2,self. y - 2 * self.thickness, self.thickness, 3*self.thickness )
	-- show current setting for selected scrollbar parameter
	gr.print(math.floor(100*self.table[sel].cur)/100,self.x+self.length+self.fontsize,self.y-12)
	-- description of parameter being controlled by scrollbar
	gr.printf( self.table[sel][1], self.x, self.y + 5, self.length, 'center' )
	end

function scrollbar:mouse()
	local mx, my	= love.mouse.getPosition( )
	local color		= self.color.slide
	if love.mouse.isDown( 'l' ) then						-- if clicked, take action
		if mx < self.x + self.length and mx > self.x then	-- mouse must be over scrollbar (x only)
			color		= self.color.sel					-- change color to show change in progress
			self.table[sel].cur = ( mx - self.x ) * (self.table[sel].max-self.table[sel].min) / self.length+self.table[sel].min
			self.table[sel][2]( self.table[sel].cur )		-- provide current setting to action function
			end
		else
		if self:mouse_over_slider( mx, my ) then			-- show mouse over slider
			color = self.color.over
			end
		end
	return color
	end

return scrollbar
